# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2013, 2015, 2016 CERN.
#
# Invenio is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# Invenio is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Invenio; if not, write to the Free Software Foundation, Inc.,
# 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
#
# In applying this license, CERN does not
# waive the privileges and immunities granted to it by virtue of its status
# as an Intergovernmental Organization or submit itself to any jurisdiction.

"""Utils for communities."""

from __future__ import absolute_import, print_function

import os
from math import ceil
from uuid import UUID

from flask import current_app
from invenio_db import db
from invenio_files_rest.errors import FilesException
from invenio_files_rest.models import Bucket, Location, ObjectVersion
from invenio_records.api import Record


class Pagination(object):
    """Helps with rendering pagination list."""

    def __init__(self, page, per_page, total_count):
        """Init pagination."""
        self.page = page
        self.per_page = per_page
        self.total_count = total_count

    @property
    def pages(self):
        """Return number of pages."""
        return int(ceil(self.total_count / float(self.per_page)))

    @property
    def has_prev(self):
        """Return true if it has previous page."""
        return self.page > 1

    @property
    def has_next(self):
        """Return true if it has next page."""
        return self.page < self.pages

    def iter_pages(self, left_edge=1, left_current=1, right_current=3,
                   right_edge=1):
        """Iterate the pages."""
        last = 0
        for num in range(1, self.pages + 1):
            if num <= left_edge or \
               (num > self.page - left_current - 1 and
                num < self.page + right_current) or \
               num > self.pages - right_edge:
                if last + 1 != num:
                    yield None
                yield num
                last = num


def render_template_to_string(input, _from_string=False, **context):
    """Render a template from the template folder with the given context.

    Code based on
    `<https://github.com/mitsuhiko/flask/blob/master/flask/templating.py>`_
    :param input: the string template, or name of the template to be
                  rendered, or an iterable with template names
                  the first one existing will be rendered
    :param context: the variables that should be available in the
                    context of the template.
    :return: a string
    """
    if _from_string:
        template = current_app.jinja_env.from_string(input)
    else:
        template = current_app.jinja_env.get_or_select_template(input)
    return template.render(context)


def save_and_validate_logo(logo_stream, logo_filename, community_id):
    """Validate if communities logo is in limit size and save it."""
    cfg = current_app.config

    logos_bucket_id = cfg['COMMUNITIES_BUCKET_UUID']
    logos_bucket = Bucket.query.get(logos_bucket_id)
    ext = os.path.splitext(logo_filename)[1]
    ext = ext[1:] if ext.startswith('.') else ext

    if ext in cfg['COMMUNITIES_LOGO_EXTENSIONS']:
        key = "{0}/logo.{1}".format(community_id, ext)
        ObjectVersion.create(logos_bucket, key, stream=logo_stream)
        return ext
    else:
        return None


def initialize_communities_bucket():
    """Initialize the communities file bucket.

    :raises: `invenio_files_rest.errors.FilesException`
    """
    bucket_id = UUID(current_app.config['COMMUNITIES_BUCKET_UUID'])

    if Bucket.query.get(bucket_id):
        raise FilesException("Bucket with UUID {} already exists.".format(
            bucket_id))
    else:
        storage_class = current_app.config['FILES_REST_DEFAULT_STORAGE_CLASS']
        location = Location.get_default()
        bucket = Bucket(id=bucket_id,
                        location=location,
                        default_storage_class=storage_class)
        db.session.add(bucket)
        db.session.commit()


def format_request_email_templ(increq, template, **ctx):
    """Format the email message element for inclusion request notification.

    Formats the message according to the provided template file, using
    some default fields from 'increq' object as default context.
    Arbitrary context can be provided as keywords ('ctx'), and those will
    not be overwritten by the fields from 'increq' object.

    :param increq: Inclusion request object for which the request is made.
    :type increq: `invenio_communities.models.InclusionRequest`
    :param template: relative path to jinja template.
    :type template: str
    :param ctx: Optional extra context parameters passed to formatter.
    :type ctx: dict.
    :returns: Formatted message.
    :rtype: str
    """
    # Add minimal information to the contex (without overwriting).
    curate_link = '{site_url}/communities/{id}/curate/'.format(
        site_url=current_app.config['THEME_SITEURL'],
        id=increq.community.id),

    min_ctx = dict(
        record=Record.get_record(increq.record.id),
        requester=increq.user,
        community=increq.community,
        curate_link=curate_link,
    )
    for k, v in min_ctx.items():
        if k not in ctx:
            ctx[k] = v

    msg_element = render_template_to_string(template, **ctx)
    return msg_element


def format_request_email_title(increq, **ctx):
    """Format the email message title for inclusion request notification.

    :param increq: Inclusion request object for which the request is made.
    :type increq: `invenio_communities.models.InclusionRequest`
    :param ctx: Optional extra context parameters passed to formatter.
    :type ctx: dict.
    :returns: Email message title.
    :rtype: str
    """
    template = current_app.config["COMMUNITIES_REQUEST_EMAIL_TITLE_TEMPLATE"],
    return format_request_email_templ(increq, template, **ctx)


def format_request_email_body(increq, **ctx):
    """Format the email message body for inclusion request notification.

    :param increq: Inclusion request object for which the request is made.
    :type increq: `invenio_communities.models.InclusionRequest`
    :param ctx: Optional extra context parameters passed to formatter.
    :type ctx: dict.
    :returns: Email message body.
    :rtype: str
    """
    template = current_app.config["COMMUNITIES_REQUEST_EMAIL_BODY_TEMPLATE"],
    return format_request_email_templ(increq, template, **ctx)


def send_community_request_email(increq):
    """Signal for sending emails after community inclusion request."""
    from flask_mail import Message
    from invenio_mail.tasks import send_email

    msg_body = format_request_email_body(increq)
    msg_title = format_request_email_title(increq)

    sender = current_app.config['COMMUNITIES_REQUEST_EMAIL_SENDER']

    msg = Message(
        msg_title,
        sender=sender,
        recipients=[increq.community.owner.email, ],
        body=msg_body
    )

    send_email.delay(msg.__dict__)



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2013, 2014, 2015, 2016 CERN.
#
# Invenio is free software; you can redistribute it
# and/or modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# Invenio is distributed in the hope that it will be
# useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Invenio; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA 02111-1307, USA.
#
# In applying this license, CERN does not
# waive the privileges and immunities granted to it by virtue of its status
# as an Intergovernmental Organization or submit itself to any jurisdiction.

"""Forms for communities."""

from __future__ import absolute_import, print_function

from flask_babelex import gettext as _
from flask_wtf import Form
from wtforms import FileField, HiddenField, StringField, TextAreaField, \
    validators

from .models import Community


class CommunityForm(Form):
    """Community form."""

    field_sets = [
        ('Information',
         ['identifier', 'title', 'description', 'curation_policy', 'page',
          'logo', ],
         {'classes': 'in'}),
    ]

    field_placeholders = {
    }

    field_state_mapping = {
    }

    #
    # Methods
    #
    def get_field_icon(self, name):
        """Return field icon."""
        return self.field_icons.get(name, '')

    def get_field_by_name(self, name):
        """Return field by name."""
        try:
            return self._fields[name]
        except KeyError:
            return None

    def get_field_placeholder(self, name):
        """Return field placeholder."""
        return self.field_placeholders.get(name, "")

    def get_field_state_mapping(self, field):
        """Return field state mapping."""
        try:
            return self.field_state_mapping[field.short_name]
        except KeyError:
            return None

    def has_field_state_mapping(self, field):
        """Check if field has state mapping."""
        return field.short_name in self.field_state_mapping

    def has_autocomplete(self, field):
        """Check if filed has autocomplete."""
        return hasattr(field, 'autocomplete')

    #
    # Fields
    #
    identifier = StringField(
        label=_('Identifier'),
        description=_('Required. Only letters, numbers and dash are allowed.'
                      ' The identifier is used in the URL for the community'
                      ' collection, and cannot be modified later.'),
        validators=[validators.DataRequired(),
                    validators.length(
                        max=100,
                        message=_("The identifier must be less"
                                  " than 100 characters long.")),
                    validators.regexp(
                        u'^[-\w]+$',
                        message=_(
                            'Only letters, numbers and dash are allowed'))]
    )

    title = StringField(
        description=_('Required.'),
        validators=[validators.DataRequired()]
    )

    description = TextAreaField(
        description=_(
            'Optional. A short description of the community collection,'
            ' which will be displayed on the index page of the community.'),
    )

    curation_policy = TextAreaField(
        description=_(
            'Optional. Please describe short and precise the policy by which'
            ' you accepted/reject new uploads in this community.'),
    )

    page = TextAreaField(
        description=_(
            'Optional. A long description of the community collection,'
            ' which will be displayed on a separate page linked from'
            ' the index page.'),
    )

    field_icons = {
        'identifier': 'barcode',
        'title': 'file-alt',
        'description': 'pencil',
        'curation_policy': 'check',
    }

    logo = FileField(
        label=_('Logo'),
        description=_(
            "Optional. Image file used to aid and promote instant public"
            " recognition. Supported formats: png and jpg."
            " Max file size: 1.5MB")
    )

    #
    # Validation
    #
    def validate_identifier(self, field):
        """Validate field identifier."""
        if field.data:
            field.data = field.data.lower()
            if Community.get(field.data, with_deleted=True):
                raise validators.ValidationError(
                    _("The identifier already exists. "
                      "Please choose a different one."))


class EditCommunityForm(CommunityForm):
    """Edit community form.

    Same as collection form, except identifier is removed.
    """

    identifier = None


class DeleteCommunityForm(Form):
    """Confirm deletion of a collection."""

    delete = HiddenField(default='yes', validators=[validators.DataRequired()])


class SearchForm(Form):
    """Search Form."""

    p = StringField(
        validators=[validators.DataRequired()]
    )



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2016 CERN.
#
# Invenio is free software; you can redistribute it
# and/or modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# Invenio is distributed in the hope that it will be
# useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Invenio; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA 02111-1307, USA.
#
# In applying this license, CERN does not
# waive the privileges and immunities granted to it by virtue of its status
# as an Intergovernmental Organization or submit itself to any jurisdiction.

"""Invenio module that adds API to communities."""

from __future__ import absolute_import, print_function

from flask import Blueprint, abort
from invenio_rest import ContentNegotiatedMethodView
from webargs import fields
from webargs.flaskparser import use_kwargs

from invenio_communities.links import default_links_item_factory, \
    default_links_pagination_factory
from invenio_communities.models import Community
from invenio_communities.serializers import community_response

blueprint = Blueprint(
    'invenio_communities_rest',
    __name__,
    url_prefix='/communities',
    template_folder='../templates',
    static_folder='../static',
)


class CommunitiesResource(ContentNegotiatedMethodView):
    """"Communities resource."""

    get_args = dict(
        query=fields.String(
            location='query',
            load_from='q',
            missing=None,
        ),
        sort=fields.String(
            location='query',
            missing=None,
        ),
        page=fields.Int(
            location='query',
            missing=1,
        ),
        size=fields.Int(
            location='query',
            missing=20,
        )
    )

    def __init__(self, serializers=None, *args, **kwargs):
        """Constructor."""
        super(CommunitiesResource, self).__init__(
            serializers,
            *args,
            **kwargs
        )

    @use_kwargs(get_args)
    def get(self, query, sort, page, size):
        """Get a list of all the communities.

        .. http:get:: /communities/(string:id)
            Returns a JSON list with all the communities.
            **Request**:
            .. sourcecode:: http
                GET /communities HTTP/1.1
                Accept: application/json
                Content-Type: application/json
                Host: localhost:5000
            :reqheader Content-Type: application/json
            **Response**:
            .. sourcecode:: http
                HTTP/1.0 200 OK
                Content-Length: 334
                Content-Type: application/json
                [
                    {
                        "id": "comm1"
                    },
                    {
                        "id": "comm2"
                    }
                ]
            :resheader Content-Type: application/json
            :statuscode 200: no error
        """
        urlkwargs = {
            'q': query,
            'sort': sort,
            'size': size,
        }

        communities = Community.filter_communities(query, sort)
        page = communities.paginate(page, size)

        links = default_links_pagination_factory(page, urlkwargs)

        links_headers = map(lambda key: ('link', 'ref="{0}" href="{1}"'.format(
            key, links[key])), links)

        return self.make_response(
            page,
            headers=links_headers,
            links_item_factory=default_links_item_factory,
            page=page,
            urlkwargs=urlkwargs,
            links_pagination_factory=default_links_pagination_factory,
        )


class CommunityDetailsResource(ContentNegotiatedMethodView):
    """"Community details resource."""

    def __init__(self, serializers=None, *args, **kwargs):
        """Constructor."""
        super(CommunityDetailsResource, self).__init__(
            serializers,
            *args,
            **kwargs
        )

    def get(self, community_id):
        """Get the details of the specified community.

        .. http:get:: /communities/(string:id)
            Returns a JSON dictionary with the details of the specified
            community.
            **Request**:
            .. sourcecode:: http
                GET /communities/communities/comm1 HTTP/1.1
                Accept: application/json
                Content-Type: application/json
                Host: localhost:5000
            :reqheader Content-Type: application/json
            :query string id: ID of an specific community to get more
                              information.
            **Response**:
            .. sourcecode:: http
                HTTP/1.0 200 OK
                Content-Length: 334
                Content-Type: application/json
                {
                    "id_user": 1,
                    "description": "",
                    "title": "",
                    "created": "2016-04-05T14:56:37.051462",
                    "id": "comm1",
                    "page": "",
                    "curation_policy": ""
                }

            :resheader Content-Type: application/json
            :statuscode 200: no error
            :statuscode 404: page not found
        """
        community = Community.get(community_id)
        if not community:
            abort(404)
        etag = community.version_id
        self.check_etag(etag)
        response = self.make_response(
            community, links_item_factory=default_links_item_factory)
        response.set_etag(etag)
        return response

serializers = {'application/json': community_response}


blueprint.add_url_rule(
    '/',
    view_func=CommunitiesResource.as_view(
        'communities_list',
        serializers=serializers,
        default_media_type='application/json',
    ),
    methods=['GET']
)


blueprint.add_url_rule(
    '/<string:community_id>',
    view_func=CommunityDetailsResource.as_view(
        'communities_item',
        serializers=serializers,
        default_media_type='application/json',
    ),
    methods=['GET']
)



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2016 CERN.
#
# Invenio is free software; you can redistribute it
# and/or modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# Invenio is distributed in the hope that it will be
# useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Invenio; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA 02111-1307, USA.
#
# In applying this license, CERN does not
# waive the privileges and immunities granted to it by virtue of its status
# as an Intergovernmental Organization or submit itself to any jurisdiction.


"""Pytest configuration."""

from __future__ import absolute_import, print_function

import os
import shutil
import tempfile

import pytest
from flask import Flask
from flask_babelex import Babel
from flask_celeryext import FlaskCeleryExt
from flask_cli import FlaskCLI
from flask_menu import Menu
from invenio_accounts import InvenioAccounts
from invenio_accounts.testutils import create_test_user
from invenio_assets import InvenioAssets
from invenio_db import db as db_
from invenio_db import InvenioDB
from invenio_indexer import InvenioIndexer
from invenio_mail import InvenioMail
from invenio_oaiserver import InvenioOAIServer
from invenio_records import InvenioRecords
from invenio_search import InvenioSearch
from sqlalchemy_utils.functions import create_database, database_exists

from invenio_communities import InvenioCommunities
from invenio_communities.models import Community
from invenio_communities.views.api import blueprint as api_blueprint
from invenio_communities.views.ui import blueprint as ui_blueprint


@pytest.yield_fixture()
def app(request):
    """Flask application fixture."""
    instance_path = tempfile.mkdtemp()
    app = Flask('testapp', instance_path=instance_path)
    app.config.update(
        TESTING=True,
        CELERY_ALWAYS_EAGER=True,
        CELERY_CACHE_BACKEND="memory",
        CELERY_EAGER_PROPAGATES_EXCEPTIONS=True,
        CELERY_RESULT_BACKEND="cache",
        SECRET_KEY='CHANGE_ME',
        SECURITY_PASSWORD_SALT='CHANGE_ME_ALSO',
        SQLALCHEMY_DATABASE_URI=os.environ.get(
            'SQLALCHEMY_DATABASE_URI', 'sqlite:///test.db'),
        SEARCH_ELASTIC_HOSTS=os.environ.get(
            'SEARCH_ELASTIC_HOSTS', None),
        SQLALCHEMY_TRACK_MODIFICATIONS=True,
        OAISERVER_REGISTER_RECORD_SIGNALS=True,
        OAISERVER_REGISTER_SET_SIGNALS=False,
        OAISERVER_ID_PREFIX='oai:localhost:recid/',
        SERVER_NAME='inveniosoftware.org',
        THEME_SITEURL='https://localhost:5000',
        MAIL_SUPPRESS_SEND=True,
    )
    FlaskCLI(app)
    FlaskCeleryExt(app)
    Menu(app)
    Babel(app)
    InvenioDB(app)
    InvenioAccounts(app)
    InvenioAssets(app)
    InvenioSearch(app)
    InvenioRecords(app)
    InvenioIndexer(app)
    InvenioOAIServer(app)
    InvenioCommunities(app)
    InvenioMail(app)

    app.register_blueprint(ui_blueprint)
    app.register_blueprint(api_blueprint, url_prefix='/api/communities')

    with app.app_context():
        yield app

    shutil.rmtree(instance_path)


@pytest.yield_fixture()
def db(app):
    """Database fixture."""
    if not database_exists(str(db_.engine.url)) and \
            app.config['SQLALCHEMY_DATABASE_URI'] != 'sqlite://':
        create_database(db_.engine.url)
    db_.create_all()

    yield db_

    db_.session.remove()
    db_.drop_all()


@pytest.fixture()
def user():
    """Create a example user."""
    return create_test_user()


@pytest.fixture()
def communities(app, db, user):
    """Create some example communities."""
    user1 = db_.session.merge(user)

    comm0 = Community.create(community_id='comm1', user_id=user1.id,
                             title='Title1', description='Description1')
    comm1 = Community.create(community_id='comm2', user_id=user1.id, title='A')
    comm2 = Community.create(community_id='oth3', user_id=user1.id)
    return comm0, comm1, comm2



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2013, 2014, 2015, 2016 CERN.
#
# Invenio is free software; you can redistribute it
# and/or modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# Invenio is distributed in the hope that it will be
# useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Invenio; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA 02111-1307, USA.
#
# In applying this license, CERN does not
# waive the privileges and immunities granted to it by virtue of its status
# as an Intergovernmental Organization or submit itself to any jurisdiction.

"""Invenio-Communities database models."""

from __future__ import absolute_import, print_function

import hashlib
from datetime import datetime

from flask import current_app, url_for
from invenio_accounts.models import User
from invenio_db import db
from invenio_records.api import Record
from invenio_records.models import RecordMetadata
from sqlalchemy.exc import IntegrityError
from sqlalchemy.orm.exc import FlushError
from sqlalchemy_utils.models import Timestamp
from sqlalchemy_utils.types import UUIDType

from .errors import CommunitiesError, InclusionRequestExistsError, \
    InclusionRequestExpiryTimeError, InclusionRequestMissingError, \
    InclusionRequestObsoleteError
from .signals import inclusion_request_created
from .utils import save_and_validate_logo


class InclusionRequest(db.Model, Timestamp):
    """Association table for Community and Record models.

    A many-to-many association table for records waiting for community
    acceptance or rejection.
    """

    __tablename__ = 'communities_community_record'

    id_community = db.Column(
        db.String(100),
        db.ForeignKey("communities_community.id"),
        primary_key=True
    )
    """Id of the community to which the record is applying."""

    id_record = db.Column(
        UUIDType,
        db.ForeignKey(RecordMetadata.id),
        primary_key=True
    )
    """Id of the record applying to given community."""

    id_user = db.Column(
        db.Integer,
        db.ForeignKey(User.id),
        nullable=True,
        default=None
    )
    """User making the request (optional)."""

    expires_at = db.Column(
        db.DateTime,
        nullable=True,
        default=None,
    )
    """Expiry date of the record request."""

    #
    # Relationships
    #
    community = db.relationship(
        'Community', backref='inclusion_requests', foreign_keys=[id_community])
    """Relation to the community to which the inclusion request is made."""

    record = db.relationship(
        RecordMetadata, backref='inclusion_requests', foreign_keys=[id_record])
    """Relation to the record which is requesting for community inclusion."""

    user = db.relationship(
        User, backref='inclusion_requests', foreign_keys=[id_user])
    """Relation to the User making the inclusion request."""

    def get_record(self):
        """Return the API object for the Record."""
        return Record.get_record(self.id_record)

    def delete(self):
        """Delete this request."""
        db.session.delete(self)

    @classmethod
    def create(cls, community, record, user=None, expires_at=None,
               notify=True):
        """Create a record inclusion request to a community.

        :param community: Community object.
        :param record: Record API object.
        :param expires_at: Time after which the request expires and shouldn't
            be resolved anymore.
        """
        if expires_at and expires_at < datetime.utcnow():
            raise InclusionRequestExpiryTimeError(
                community=community, record=record)

        if community.has_record(record):
            raise InclusionRequestObsoleteError(
                community=community, record=record)

        try:
            # Create inclusion request
            with db.session.begin_nested():
                obj = cls(
                    id_community=community.id,
                    id_record=record.id,
                    user=user,
                    expires_at=expires_at
                )
                db.session.add(obj)
        except (IntegrityError, FlushError):
            raise InclusionRequestExistsError(
                community=community, record=record)

        # Send signal
        inclusion_request_created.send(
            current_app._get_current_object(),
            request=obj,
            notify=notify
        )

        return obj

    @classmethod
    def get(cls, community_id, record_uuid):
        """Get an inclusion request."""
        return cls.query.filter_by(
            id_record=record_uuid, id_community=community_id
        ).one_or_none()

    @classmethod
    def get_by_record(cls, record_uuid):
        """Get inclusion requests for a given record."""
        return cls.query.filter_by(id_record=record_uuid)


class Community(db.Model, Timestamp):
    """Represent a community."""

    __tablename__ = 'communities_community'

    id = db.Column(db.String(100), primary_key=True)
    """Id of the community."""

    id_user = db.Column(
        db.Integer,
        db.ForeignKey(User.id),
        nullable=False
    )
    """Owner of the community."""

    title = db.Column(db.String(length=255), nullable=False, default='')
    """Title of the community."""

    description = db.Column(db.Text, nullable=False, default='')
    """Short description of community, displayed in portal boxes."""

    page = db.Column(db.Text, nullable=False, default='')
    """Long description of community, displayed on an individual page."""

    curation_policy = db.Column(db.Text(), nullable=False, default='')
    """Community curation policy."""

    last_record_accepted = db.Column(
        db.DateTime(), nullable=False, default=datetime(2000, 1, 1, 0, 0, 0))
    """Last record acceptance datetime."""

    logo_ext = db.Column(db.String(length=4), nullable=True, default=None)
    """Extension of the logo."""

    ranking = db.Column(db.Integer, nullable=False, default=0)
    """Ranking of community. Updated by ranking deamon."""

    fixed_points = db.Column(db.Integer, nullable=False, default=0)
    """Points which will be always added to overall score of community."""

    deleted_at = db.Column(db.DateTime, nullable=True, default=None)
    """Time at which the community was soft-deleted."""

    #
    # Relationships
    #
    owner = db.relationship(User, backref='communities',
                            foreign_keys=[id_user])
    """Relation to the owner (User) of the community."""

    def __repr__(self):
        """String representation of the community object."""
        return "<Community, ID: {}>".format(self.id)

    @classmethod
    def create(cls, community_id, user_id, **data):
        """Get a community."""
        with db.session.begin_nested():
            obj = cls(id=community_id, id_user=user_id, **data)
            db.session.add(obj)
        return obj

    def save_logo(self, stream, filename):
        """Get a community."""
        logo_ext = save_and_validate_logo(stream, filename, self.id)
        if logo_ext:
            self.logo_ext = logo_ext
            return True
        return False

    @classmethod
    def get(cls, community_id, with_deleted=False):
        """Get a community."""
        q = cls.query.filter_by(id=community_id)
        if not with_deleted:
            q = q.filter(cls.deleted_at.is_(None))
        return q.one_or_none()

    @classmethod
    def get_by_user(cls, user_id, with_deleted=False):
        """Get a community."""
        query = cls.query.filter_by(
            id_user=user_id
        )
        if not with_deleted:
            query = query.filter(cls.deleted_at.is_(None))

        return query.order_by(db.asc(Community.title))

    @classmethod
    def filter_communities(cls, p, so, with_deleted=False):
        """Search for communities.

        Helper function which takes from database only those communities which
        match search criteria. Uses parameter 'so' to set communities in the
        correct order.

        Parameter 'page' is introduced to restrict results and return only
        slice of them for the current page. If page == 0 function will return
        all communities that match the pattern.
        """
        query = cls.query if with_deleted else \
            cls.query.filter(cls.deleted_at.is_(None))

        if p:
            query = query.filter(db.or_(
                cls.id.like("%" + p + "%"),
                cls.title.like("%" + p + "%"),
                cls.description.like("%" + p + "%"),
            ))

        if so in current_app.config['COMMUNITIES_SORTING_OPTIONS']:
            order = so == 'title' and db.asc or db.desc
            query = query.order_by(order(getattr(cls, so)))
        else:
            query = query.order_by(db.desc(cls.ranking))
        return query

    def add_record(self, record):
        """Add a record to the community.

        :param record: Record object.
        :type record: `invenio_records.api.Record`
        """
        key = current_app.config['COMMUNITIES_RECORD_KEY']
        record.setdefault(key, [])

        assert self.id not in record[key]
        record[key].append(self.id)
        record[key] = sorted(record[key])

        if current_app.config["COMMUNITIES_OAI_ENABLED"]:
            from invenio_oaiserver.models import OAISet
            oaiset = OAISet.query.filter_by(spec=self.oaiset_spec).one()
            oaiset.add_record(record)

    def remove_record(self, record):
        """Remove an already accepted record from the community.

        :param record: Record object.
        :type record: `invenio_records.api.Record`
        """
        key = current_app.config['COMMUNITIES_RECORD_KEY']

        assert self.id in record.get(key, [])

        record[key] = [c for c in record[key] if c != self.id]

        if current_app.config["COMMUNITIES_OAI_ENABLED"]:
            from invenio_oaiserver.models import OAISet
            oaiset = OAISet.query.filter_by(spec=self.oaiset_spec).one()
            oaiset.remove_record(record)

    def has_record(self, record):
        """Check if record is in community."""
        return self.id in \
            record.get(current_app.config["COMMUNITIES_RECORD_KEY"], [])

    def accept_record(self, record):
        """Accept a record for inclusion in the community.

        :param record: Record object.
        """
        with db.session.begin_nested():
            req = InclusionRequest.get(self.id, record.id)
            if req is None:
                raise InclusionRequestMissingError(community=self,
                                                   record=record)
            req.delete()
            self.add_record(record)
            self.last_record_accepted = datetime.utcnow()

    def reject_record(self, record):
        """Reject a record for inclusion in the community.

        :param record: Record object.
        """
        with db.session.begin_nested():
            req = InclusionRequest.get(self.id, record.id)
            if req is None:
                raise InclusionRequestMissingError(community=self,
                                                   record=record)
            req.delete()

    def delete(self):
        """Mark the community for deletion.

        :param delete_time: DateTime after which to delete the community.
        :type delete_time: datetime.datetime
        :raises: CommunitiesError
        """
        if self.deleted_at is not None:
            raise CommunitiesError(community=self)
        else:
            self.deleted_at = datetime.utcnow()

    def undelete(self):
        """Remove the community marking for deletion."""
        if self.deleted_at is None:
            raise CommunitiesError(community=self)
        else:
            self.deleted_at = None

    @property
    def is_deleted(self):
        """Return whether given community is marked for deletion."""
        return self.deleted_at is not None

    @property
    def logo_url(self):
        """Get URL to collection logo.

        :returns: Path to community logo.
        :rtype: str
        """
        if self.logo_ext:
            buc = current_app.config['COMMUNITIES_BUCKET_UUID']
            key = "{0}/logo.{1}".format(self.id, self.logo_ext)
            return url_for(
                'invenio_files_rest.object_api', bucket_id=buc, key=key)
        else:
            return None

    @property
    def community_url(self):
        """Get provisional URL."""
        return url_for(
            'invenio_communities.detail', community_id=self.id, _external=True)

    @property
    def community_provisional_url(self):
        """Get provisional URL."""
        return url_for(
            'invenio_communities.curate', community_id=self.id, _external=True)

    @property
    def upload_url(self):
        """Get provisional URL."""
        return 'TODO'

    @property
    def oaiset_spec(self):
        """Return the OAISet 'spec' name for given community.

        :returns: name of corresponding OAISet ('spec').
        :rtype: str
        """
        return current_app.config['COMMUNITIES_OAI_FORMAT'].format(
            community_id=self.id)

    @property
    def oaiset_url(self):
        """Return the OAISet 'spec' name for given community.

        :returns: name of corresponding OAISet ('spec').
        :rtype: str
        """
        return url_for(
            'invenio_oaiserver.response',
            verb='ListRecords',
            metadataPrefix='oai_dc', set=self.oaiset_spec, _external=True)

    @property
    def version_id(self):
        """Return the version of the community.

        :returns: hash which encodes the community id and its las update.
        :rtype: str
        """
        return hashlib.sha1('{0}__{1}'.format(
            self.id, self.updated).encode('utf-8')).hexdigest()


class FeaturedCommunity(db.Model, Timestamp):
    """Represent a featured community."""

    __tablename__ = 'communities_featured_community'

    id = db.Column(db.Integer, primary_key=True)
    """Id of the featured entry."""

    id_community = db.Column(
        db.String(100), db.ForeignKey(Community.id), nullable=False)
    """Id of the featured community."""

    start_date = db.Column(
        db.DateTime, nullable=False, default=datetime.utcnow)
    """Start date of the community featuring."""

    #
    # Relationships
    #
    community = db.relationship(Community, backref="featuredcommunity")
    """Relation to the community."""

    @classmethod
    def get_featured_or_none(cls, start_date=None):
        """Get the latest featured community.

        :param start_date: Date after which the featuring starts
        :returns: Community object or None
        :rtype: `invenio_communities.models.Community` or None
        """
        start_date = start_date or datetime.utcnow()

        comm = cls.query.filter(
            FeaturedCommunity.start_date <= start_date
        ).order_by(
            cls.start_date.desc()
        ).first()
        return comm if comm is None else comm.community



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2016 CERN.
#
# Invenio is free software; you can redistribute it
# and/or modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# Invenio is distributed in the hope that it will be
# useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Invenio; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA 02111-1307, USA.
#
# In applying this license, CERN does not
# waive the privileges and immunities granted to it by virtue of its status
# as an Intergovernmental Organization or submit itself to any jurisdiction.


"""Utility functions tests."""

from __future__ import absolute_import, print_function

from invenio_records.api import Record

from invenio_communities.models import InclusionRequest
from invenio_communities.utils import render_template_to_string


def test_template_formatting_from_string(app):
    """Test formatting of string-based template to string."""
    with app.app_context():
        out = render_template_to_string("foobar: {{ baz }}", _from_string=True,
                                        **{'baz': 'spam'})
        assert out == 'foobar: spam'


def test_email_formatting(app, db, communities, user):
    """Test formatting of the email message with the default template."""
    with app.extensions['mail'].record_messages() as outbox:
        (comm1, comm2, comm3) = communities
        rec1 = Record.create({
            'title': 'Foobar and Bazbar',
            'description': 'On Foobar, Bazbar and <b>more</b>.'
        })

        # Request
        InclusionRequest.create(community=comm1, record=rec1, user=user)

        # Check emails being sent
        assert len(outbox) == 1
        sent_msg = outbox[0]
        assert sent_msg.recipients == [user.email]
        assert comm1.title in sent_msg.body



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2016 CERN.
#
# Invenio is free software; you can redistribute it
# and/or modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# Invenio is distributed in the hope that it will be
# useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Invenio; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA 02111-1307, USA.
#
# In applying this license, CERN does not
# waive the privileges and immunities granted to it by virtue of its status
# as an Intergovernmental Organization or submit itself to any jurisdiction.

"""Community JSON schema."""

from __future__ import absolute_import, print_function

from marshmallow import Schema, fields, post_dump

from invenio_communities.links import default_links_item_factory, \
    default_links_pagination_factory


class CommunitySchemaV1(Schema):
    """Schema for a community."""

    id = fields.String()
    title = fields.String()
    description = fields.String()
    page = fields.String()
    curation_policy = fields.String()
    last_record_accepted = fields.DateTime()

    @post_dump(pass_many=False)
    def item_links_addition(self, data):
        """Add the links for each community."""
        links_item_factory = self.context.get('links_item_factory',
                                              default_links_item_factory)
        data['links'] = links_item_factory(data)
        return data

    @post_dump(pass_many=True)
    def envelope(self, data, many):
        """Wrap result in envelope."""
        if not many:
            return data

        result = dict(
            hits=dict(
                hits=data,
                total=self.context.get('total', len(data))
            )
        )

        page = self.context.get('page')
        if page:
            links_pagination_factory = self.context.get(
                'links_pagination_factory',
                default_links_pagination_factory
            )

            urlkwargs = self.context.get('urlkwargs', {})

            result['links'] = links_pagination_factory(page, urlkwargs)

        return result



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2016 CERN.
#
# Invenio is free software; you can redistribute it
# and/or modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# Invenio is distributed in the hope that it will be
# useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Invenio; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA 02111-1307, USA.
#
# In applying this license, CERN does not
# waive the privileges and immunities granted to it by virtue of its status
# as an Intergovernmental Organization or submit itself to any jurisdiction.

"""Click command-line interface for communities management."""

from __future__ import absolute_import, print_function

import click
from flask_cli import with_appcontext
from invenio_db import db
from invenio_files_rest.errors import FilesException
from invenio_indexer.api import RecordIndexer
from invenio_records.api import Record

from .models import Community, InclusionRequest
from .utils import initialize_communities_bucket, save_and_validate_logo


#
# Communities management commands
#
@click.group()
def communities():
    """Management commands for Communities."""


@communities.command()
@with_appcontext
def init():
    """Initialize the communities file storage."""
    try:
        initialize_communities_bucket()
        click.secho('Community init successful.', fg='green')
    except FilesException as e:
        click.secho(e.message, fg='red')


@communities.command()
@click.argument('community_id')
@click.argument('logo', type=click.File('rb'))
@with_appcontext
def addlogo(community_id, logo):
    """Add logo to the community."""
    # Create the bucket
    c = Community.get(community_id)
    if not c:
        click.secho('Community {0} does not exist.'.format(community_id),
                    fg='red')
        return
    ext = save_and_validate_logo(logo, logo.name, c.id)
    c.logo_ext = ext
    db.session.commit()


@communities.command()
@click.argument('community_id')
@click.argument('record_id')
@click.option('-a', '--accept', 'accept', is_flag=True, default=False)
@with_appcontext
def request(community_id, record_id, accept):
    """Request a record acceptance to a community."""
    c = Community.get(community_id)
    assert c is not None
    record = Record.get_record(record_id)
    if accept:
        c.add_record(record)
        record.commit()
    else:
        InclusionRequest.create(community=c, record=record,
                                notify=False)
    db.session.commit()
    RecordIndexer().index_by_id(record.id)


@communities.command()
@click.argument('community_id')
@click.argument('record_id')
@with_appcontext
def remove(community_id, record_id):
    """Remove a record from community."""
    c = Community.get(community_id)
    assert c is not None
    c.remove_record(record_id)
    db.session.commit()
    RecordIndexer().index_by_id(record_id)



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2016 CERN.
#
# Invenio is free software; you can redistribute it
# and/or modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# Invenio is distributed in the hope that it will be
# useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Invenio; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA 02111-1307, USA.
#
# In applying this license, CERN does not
# waive the privileges and immunities granted to it by virtue of its status
# as an Intergovernmental Organization or submit itself to any jurisdiction.

"""Invenio module that adds support for communities."""

from __future__ import absolute_import, print_function

import copy
from functools import wraps

from flask import Blueprint, abort, current_app, flash, jsonify, redirect, \
    render_template, request, url_for
from flask_babelex import gettext as _
from flask_login import current_user, login_required
from invenio_db import db
from invenio_indexer.api import RecordIndexer
from invenio_pidstore.resolver import Resolver
from invenio_records.api import Record

from invenio_communities.forms import CommunityForm, DeleteCommunityForm, \
    EditCommunityForm, SearchForm
from invenio_communities.models import Community, FeaturedCommunity
from invenio_communities.proxies import current_permission_factory
from invenio_communities.utils import Pagination, render_template_to_string

blueprint = Blueprint(
    'invenio_communities',
    __name__,
    url_prefix='/communities',
    template_folder='../templates',
    static_folder='../static',
)


def pass_community(f):
    """Decorator to pass community."""
    @wraps(f)
    def inner(community_id, *args, **kwargs):
        c = Community.get(community_id)
        if c is None:
            abort(404)
        return f(c, *args, **kwargs)
    return inner


def permission_required(action):
    """Decorator to require permission."""
    def decorator(f):
        @wraps(f)
        def inner(community, *args, **kwargs):
            permission = current_permission_factory(community, action=action)
            if not permission.can():
                abort(403)
            return f(community, *args, **kwargs)
        return inner
    return decorator


@blueprint.app_template_filter('format_item')
def format_item(item, template, name='item'):
    """Render a template to a string with the provided item in context."""
    ctx = {name: item}
    return render_template_to_string(template, **ctx)


@blueprint.app_template_filter('mycommunities_ctx')
def mycommunities_ctx():
    """Helper method for return ctx used by many views."""
    return {
        'mycommunities': Community.get_by_user(current_user.get_id()).all()
    }


@blueprint.route('/', methods=['GET', ])
def index():
    """Index page with uploader and list of existing depositions."""
    ctx = mycommunities_ctx()

    p = request.args.get('p', type=str)
    so = request.args.get('so', type=str)
    page = request.args.get('page', type=int, default=1)

    so = so or current_app.config.get('COMMUNITIES_DEFAULT_SORTING_OPTION')

    communities = Community.filter_communities(p, so)
    featured_community = FeaturedCommunity.get_featured_or_none()
    form = SearchForm(p=p)
    per_page = 10
    page = max(page, 1)
    p = Pagination(page, per_page, communities.count())

    ctx.update({
        'r_from': max(p.per_page * (p.page - 1), 0),
        'r_to': min(p.per_page * p.page, p.total_count),
        'r_total': p.total_count,
        'pagination': p,
        'form': form,
        'title': _('Communities'),
        'communities': communities.slice(
            per_page * (page - 1), per_page * page).all(),
        'featured_community': featured_community,
    })

    return render_template(
        "invenio_communities/index.html",
        **ctx
    )


@blueprint.route('/<string:community_id>/', methods=['GET'])
@pass_community
def detail(community):
    """Index page with uploader and list of existing depositions."""
    return generic_item(community, "invenio_communities/detail.html")


@blueprint.route('/<string:community_id>/search', methods=['GET'])
@pass_community
def search(community):
    """Index page with uploader and list of existing depositions."""
    return generic_item(
        community,
        current_app.config['COMMUNITIES_SEARCH_TEMPLATE'],
        detail=False)


@blueprint.route('/<string:community_id>/about/', methods=['GET'])
@pass_community
def about(community):
    """Index page with uploader and list of existing depositions."""
    return generic_item(community, "invenio_communities/about.html")


def generic_item(community, template, **extra_ctx):
    """Index page with uploader and list of existing depositions."""
    # Check existence of community
    ctx = mycommunities_ctx()
    ctx.update({
        'is_owner': community.id_user == current_user.get_id(),
        'community': community,
        'detail': True,
    })
    ctx.update(extra_ctx)

    return render_template(template, **ctx)


@blueprint.route('/new/', methods=['GET', 'POST'])
@login_required
def new():
    """Create a new community."""
    form = CommunityForm(request.values)

    ctx = mycommunities_ctx()
    ctx.update({
        'form': form,
        'is_new': True,
        'community': None,
    })

    if form.validate_on_submit():
        data = copy.deepcopy(form.data)

        community_id = data.pop('identifier')
        del data['logo']

        community = Community.create(
            community_id, current_user.get_id(), **data)

        file = request.files.get('logo', None)
        if file:
            if not community.save_logo(file.stream, file.filename):
                form.logo.errors.append(_(
                    'Cannot add this file as a logo. Supported formats: '
                    '  png and jpg. Max file size: 1.5MB.'))
                db.session.rollback()
                community = None

        if community:
            db.session.commit()
            flash("Community was successfully created.", category='success')
            return redirect(url_for('.edit', community_id=community.id))

    return render_template(
        "/invenio_communities/new.html",
        community_form=form,
        **ctx
    )


@blueprint.route('/<string:community_id>/edit/', methods=['GET', 'POST'])
@login_required
@pass_community
@permission_required('community-edit')
def edit(community):
    """Create or edit a community."""
    form = EditCommunityForm(request.values, community)
    deleteform = DeleteCommunityForm()
    ctx = mycommunities_ctx()
    ctx.update({
        'form': form,
        'is_new': False,
        'community': community,
        'deleteform': deleteform,
    })

    if form.validate_on_submit():
        for field, val in form.data.items():
            setattr(community, field, val)

        file = request.files.get('logo', None)
        if file:
            if not community.save_logo(file.stream, file.filename):
                form.logo.errors.append(_(
                    'Cannot add this file as a logo. Supported formats: '
                    '  png and jpg. Max file size: 1.5MB.'))

        if not form.logo.errors:
            db.session.commit()
            flash("Community successfully edited.", category='success')
            return redirect(url_for('.edit', community_id=community.id))

    return render_template(
        "invenio_communities/new.html",
        **ctx
    )


@blueprint.route('/<string:community_id>/delete/', methods=['POST'])
@login_required
@pass_community
@permission_required('community-delete')
def delete(community):
    """Delete a community."""
    deleteform = DeleteCommunityForm(request.values)
    ctx = mycommunities_ctx()
    ctx.update({
        'deleteform': deleteform,
        'is_new': False,
        'community': community,
    })

    if deleteform.validate_on_submit():
        community.delete()
        db.session.commit()
        flash("Community was deleted.", category='success')
        return redirect(url_for('.index'))
    else:
        flash("Community could not be deleted.", category='warning')
        return redirect(url_for('.edit', community_id=community.id))


@blueprint.route('/<string:community_id>/curate/', methods=['GET', 'POST'])
@login_required
@pass_community
@permission_required('community-curate')
def curate(community):
    """Index page with uploader and list of existing depositions.

    :param community_id: ID of the community to curate.
    """
    if request.method == 'POST':
        action = request.json.get('action')
        recid = request.json.get('recid')

        # 'recid' is mandatory
        if not recid:
            abort(400)
        if action not in ['accept', 'reject', 'remove']:
            abort(400)

        # Resolve recid to a Record
        resolver = Resolver(
            pid_type='recid', object_type='rec', getter=Record.get_record)
        pid, record = resolver.resolve(recid)

        # Perform actions
        if action == "accept":
            community.accept_record(record)
        elif action == "reject":
            community.reject_record(record)
        elif action == "remove":
            community.remove_record(record)

        record.commit()
        db.session.commit()
        RecordIndexer().index_by_id(record.id)
        return jsonify({'status': 'success'})

    ctx = {'community': community}
    return render_template(
        current_app.config['COMMUNITIES_CURATE_TEMPLATE'],
        **ctx
    )



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2016 CERN.
#
# Invenio is free software; you can redistribute it
# and/or modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# Invenio is distributed in the hope that it will be
# useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Invenio; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA 02111-1307, USA.
#
# In applying this license, CERN does not
# waive the privileges and immunities granted to it by virtue of its status
# as an Intergovernmental Organization or submit itself to any jurisdiction.

"""Community module receivers."""

from __future__ import absolute_import, print_function

from flask import current_app
from invenio_db import db

from .models import InclusionRequest
from .utils import send_community_request_email


def new_request(sender, request=None, notify=True, **kwargs):
    """New request for inclusion."""
    if current_app.config['COMMUNITIES_MAIL_ENABLED'] and notify:
        send_community_request_email(request)


def inject_provisional_community(sender, json=None, record=None, index=None,
                                 **kwargs):
    """Inject 'provisional_communities' key to ES index."""
    if index and not index.startswith(
            current_app.config['COMMUNITIES_INDEX_PREFIX']):
        return

    json['provisional_communities'] = list(sorted([
        r.id_community for r in InclusionRequest.get_by_record(record.id)
    ]))


def create_oaipmh_set(mapper, connection, community):
    """Signal for creating OAI-PMH sets during community creation."""
    from invenio_oaiserver.models import OAISet
    with db.session.begin_nested():
        obj = OAISet(spec=community.oaiset_spec,
                     name=community.title,
                     description=community.description)
        db.session.add(obj)


def destroy_oaipmh_set(mapper, connection, community):
    """Signal for creating OAI-PMH sets during community creation."""
    from invenio_oaiserver.models import OAISet
    with db.session.begin_nested():
        oaiset = OAISet.query.filter_by(
            spec=community.oaiset_spec).one_or_none()
        if oaiset is None:
            raise Exception(
                "OAISet for community {0} is missing".format(community.id))
        db.session.delete(oaiset)



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2016 CERN.
#
# Invenio is free software; you can redistribute it
# and/or modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# Invenio is distributed in the hope that it will be
# useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Invenio; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA 02111-1307, USA.
#
# In applying this license, CERN does not
# waive the privileges and immunities granted to it by virtue of its status
# as an Intergovernmental Organization or submit itself to any jurisdiction.

"""Permissions for communities."""

from __future__ import absolute_import, print_function

from flask_login import current_user
from flask_principal import ActionNeed
from invenio_access.permissions import DynamicPermission


class _Permission(object):
    """Temporary solution to permissions.

    Grant access to owners of community or admin.
    TODO: Use fine-grained permissions.
    """

    def __init__(self, community, action):
        """Initialize."""
        self.community = community
        self.action = action

    def can(self):
        """Grant permission if owner or admin."""
        return str(current_user.get_id()) == str(self.community.id_user) or \
            DynamicPermission(ActionNeed('admin-access')).can()


def permission_factory(community, action):
    """Permission factory for the actions on Bucket and ObjectVersion items.

    :param community: Community object to be accessed.
    :type community: `invenio_communities.models.Community`
    :param action: Action name for access.
    :type action: str
    """
    return _Permission(community, action)



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2016 CERN.
#
# Invenio is free software; you can redistribute it
# and/or modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# Invenio is distributed in the hope that it will be
# useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Invenio; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA 02111-1307, USA.
#
# In applying this license, CERN does not
# waive the privileges and immunities granted to it by virtue of its status
# as an Intergovernmental Organization or submit itself to any jurisdiction.


"""Module tests."""

from __future__ import absolute_import, print_function

import json
from datetime import datetime, timedelta

import pytest
from flask import Flask
from flask_cli import FlaskCLI
from invenio_db import db as db_
from invenio_oaiserver.models import OAISet
from invenio_records.api import Record

from invenio_communities import InvenioCommunities
from invenio_communities.errors import CommunitiesError, \
    InclusionRequestExistsError, InclusionRequestMissingError, \
    InclusionRequestObsoleteError
from invenio_communities.models import Community, FeaturedCommunity, \
    InclusionRequest

try:
    from werkzeug.urls import url_parse
except ImportError:
    from urlparse import urlsplit as url_parse


def get_json(res, code=None):
    """Transform a JSON response into a dictionary."""
    if code:
        assert res.status_code == code
    return json.loads(res.get_data(as_text=True))


def assert_community_serialization(community, **kwargs):
    """Check the values of a community."""
    for key in kwargs.keys():
        assert community[key] == kwargs[key]


def test_version():
    """Test version import."""
    from invenio_communities import __version__
    assert __version__


def test_init():
    """Test extension initialization."""
    app = Flask('testapp')
    FlaskCLI(app)
    ext = InvenioCommunities(app)
    assert 'invenio-communities' in app.extensions

    app = Flask('testapp')
    FlaskCLI(app)
    ext = InvenioCommunities()
    assert 'invenio-communities' not in app.extensions
    ext.init_app(app)
    assert 'invenio-communities' in app.extensions


def test_model_init(app, db, communities):
    """Test basic model initialization and actions."""
    (comm1, comm2, comm3) = communities
    communities_key = app.config["COMMUNITIES_RECORD_KEY"]
    # Create a record and accept it into the community by creating an
    # InclusionRequest and then calling the accept action
    rec1 = Record.create({'title': 'Foobar'})
    InclusionRequest.create(community=comm1, record=rec1)
    assert InclusionRequest.query.count() == 1
    comm1.accept_record(rec1)
    assert 'comm1' in rec1[communities_key]
    assert InclusionRequest.query.count() == 0

    # Likewise, reject a record from the community
    rec2 = Record.create({'title': 'Bazbar'})
    InclusionRequest.create(community=comm1, record=rec2)
    assert InclusionRequest.query.count() == 1
    comm1.reject_record(rec2)
    assert communities_key not in rec2  # dict key should not be created
    assert InclusionRequest.query.count() == 0

    # Add record to another community
    InclusionRequest.create(community=comm2, record=rec1)
    comm2.accept_record(rec1)
    assert communities_key in rec1
    assert len(rec1[communities_key]) == 2
    assert comm1.id in rec1[communities_key]
    assert comm2.id in rec1[communities_key]

    # Accept/reject a record to/from a community without inclusion request
    rec3 = Record.create({'title': 'Spam'})
    pytest.raises(InclusionRequestMissingError, comm1.accept_record, rec3)
    pytest.raises(InclusionRequestMissingError, comm1.reject_record, rec3)

    # Create two inclusion requests
    InclusionRequest.create(community=comm3, record=rec1)
    db_.session.commit()
    db_.session.flush()
    pytest.raises(InclusionRequestExistsError, InclusionRequest.create,
                  community=comm3, record=rec1)

    # Try to accept a record to a community twice (should raise)
    # (comm1 is already in rec1)
    pytest.raises(InclusionRequestObsoleteError, InclusionRequest.create,
                  community=comm1, record=rec1)


def test_email_notification(app, db, communities, user):
    """Test mail notification sending for community request."""
    # Mock the send method of the Flask-Mail extension
    with app.extensions['mail'].record_messages() as outbox:
        (comm1, comm2, comm3) = communities
        # Create a record and accept it into the community by creating an
        # InclusionRequest and then calling the accept action
        rec1 = Record.create({
            'title': 'Foobar', 'description': 'Baz bar.'})
        InclusionRequest.create(community=comm1, record=rec1, user=user)
        assert len(outbox) == 1


def test_model_featured_community(app, db, communities):
    """Test the featured community model and actions."""
    (comm1, comm2, comm3) = communities
    t1 = datetime.now()

    # Create two community featurings starting at different times
    fc1 = FeaturedCommunity(id_community=comm1.id,
                            start_date=t1 + timedelta(days=1))
    fc2 = FeaturedCommunity(id_community=comm2.id,
                            start_date=t1 + timedelta(days=3))
    db_.session.add(fc1)
    db_.session.add(fc2)
    # Check the featured community at three different points in time
    assert FeaturedCommunity.get_featured_or_none(start_date=t1) is None
    assert FeaturedCommunity.get_featured_or_none(
        start_date=t1 + timedelta(days=2)) is comm1
    assert FeaturedCommunity.get_featured_or_none(
        start_date=t1 + timedelta(days=4)) is comm2


def test_oaipmh_sets(app, db, communities):
    """Test the OAI-PMH Sets creation."""
    (comm1, comm2, comm3) = communities

    assert OAISet.query.count() == 3
    oai_set1 = OAISet.query.first()
    assert oai_set1.spec == 'user-comm1'
    assert oai_set1.name == 'Title1'
    assert oai_set1.description == 'Description1'

    # Delete the community and make sure the set is also deleted
    db_.session.delete(comm1)
    db_.session.commit()
    assert Community.query.count() == 2
    assert OAISet.query.count() == 2


def test_communities_rest_all_communities(app, db, communities):
    """Test the OAI-PMH Sets creation."""
    with app.test_client() as client:
        response = client.get('/api/communities/')
        response_data = get_json(response)
        assert response_data['hits']['total'] == 3
        assert len(response_data['hits']['hits']) == 3

        assert response_data['hits']['hits'][0]['id'] == 'comm1'


def test_community_delete(app, db, communities):
    """Test deletion of communities."""
    (comm1, comm2, comm3) = communities
    comm1.delete()
    assert comm1.is_deleted is True
    comm1.undelete()
    assert comm1.is_deleted is False

    # Try to undelete a community that was not marked for deletion
    pytest.raises(CommunitiesError, comm1.undelete)

    # Try to delete community twice
    comm2.delete()
    pytest.raises(CommunitiesError, comm2.delete)


def test_communities_rest_all_communities_query_and_sort(app, db, communities):
    """Test the OAI-PMH Sets creation."""
    with app.test_client() as client:
        response = client.get('/api/communities/?q=comm&sort=title')
        response_data = get_json(response)

        assert response_data['hits']['total'] == 2
        assert response_data['hits']['hits'][0]['id'] == 'comm2'
        assert response_data['hits']['hits'][1]['id'] == 'comm1'


def test_communities_rest_pagination(app, db, communities):
    """Test the OAI-PMH Sets creation."""
    def parse_path(app, path):
        """Split the path in base and real relative url.

        Needed because in Flask 0.10.1 the client doesn't take into account the
        query string in an external URL.
        """
        http_host = app.config.get('SERVER_NAME')
        app_root = app.config.get('APPLICATION_ROOT')
        url = url_parse(path)
        base_url = 'http://{0}/'.format(url.netloc or http_host or 'localhost')
        if app_root:
            base_url += app_root.lstrip('/')
        if url.netloc:
            path = url.path
            if url.query:
                path += '?' + url.query
        return dict(path=path, base_url=base_url)

    def assert_header_links(response, ref, page, size):
        """Check if there is a pagination in one of the headers."""
        assert any(all(item in h[1] for item in [
            'ref="{0}"'.format(ref),
            'page={0}'.format(page),
            'size={0}'.format(size)]) for h in response.headers)

    with app.test_client() as client:
        response = client.get('/api/communities/?size=1')
        assert_header_links(response, 'self', 1, 1)
        assert_header_links(response, 'next', 2, 1)

        data = get_json(response)
        assert 'self' in data['links']
        assert len(data['hits']['hits']) == 1

        # Assert that self gives back the same result
        response = client.get(**parse_path(app, data['links']['self']))
        assert data == get_json(response)
        assert 'prev' not in data['links']
        assert 'next' in data['links']

        # Second page
        response = client.get(**parse_path(app, data['links']['next']))
        assert_header_links(response, 'next', 3, 1)
        assert_header_links(response, 'self', 2, 1)
        assert_header_links(response, 'prev', 1, 1)

        data = get_json(response)
        assert len(data['hits']['hits']) == 1
        assert 'prev' in data['links']
        assert 'next' in data['links']

        # Third page
        response = client.get(**parse_path(app, data['links']['next']))
        assert_header_links(response, 'self', 3, 1)
        assert_header_links(response, 'prev', 2, 1)

        data = get_json(response)
        assert 'prev' in data['links']
        assert 'next' not in data['links']


def test_communities_rest_get_details(app, db, communities):
    """Test the OAI-PMH Sets creation."""
    with app.test_client() as client:
        response = client.get('/api/communities/comm1')
        assert_community_serialization(
                get_json(response),
                description='Description1',
                title='Title1',
                id='comm1',
                page='',
                curation_policy='',
                last_record_accepted='2000-01-01T00:00:00+00:00',
                links={
                    'self': 'http://inveniosoftware.org/api/communities/comm1',
                    'html': 'http://inveniosoftware.org/communities/comm1/',
                },
        )


def test_communities_rest_etag(app, communities):
    """Test the OAI-PMH Sets creation."""
    with app.test_client() as client:
        # The first response should return the data with result code 200
        response = client.get('/api/communities/comm1')
        assert response.status_code == 200
        assert response.get_data(as_text=True) != ''

        # The second response is empty and the result code is 304
        response = client.get('/api/communities/comm1', headers=(
            ('If-None-Match', response.headers.get('ETag')),))
        assert response.status_code == 304
        assert response.get_data(as_text=True) == ''



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2016 CERN.
#
# Invenio is free software; you can redistribute it
# and/or modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# Invenio is distributed in the hope that it will be
# useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Invenio; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA 02111-1307, USA.
#
# In applying this license, CERN does not
# waive the privileges and immunities granted to it by virtue of its status
# as an Intergovernmental Organization or submit itself to any jurisdiction.


"""Module tests."""

from __future__ import absolute_import, print_function

from click.testing import CliRunner

from invenio_communities.cli import addlogo, init


def test_cli_init(script_info):
    """Test create user CLI."""
    runner = CliRunner()

    # Init a community first time.
    result = runner.invoke(init, obj=script_info)
    assert result.exit_code == 0

    # Init a community when it is already created.
    result = runner.invoke(init, obj=script_info)
    assert 'Bucket with UUID' in result.output_bytes
    assert 'already exists.\n' in result.output_bytes


def test_cli_init(script_info):
    """Test create user CLI."""
    runner = CliRunner()

    # Add logo to an unexisting community.
    result = runner.invoke(addlogo, [
        '00000000-0000-0000-0000-000000000000',
        '',
    ], obj=script_info)
    assert result.exit_code == 0

    # Add logo to an existing community.



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2015, 2016 CERN.
#
# Invenio is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# Invenio is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Invenio; if not, write to the Free Software Foundation, Inc.,
# 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
#
# In applying this license, CERN does not
# waive the privileges and immunities granted to it by virtue of its status
# as an Intergovernmental Organization or submit itself to any jurisdiction.

"""Admin model views for Communities."""

from __future__ import absolute_import, print_function

from flask_admin.contrib.sqla import ModelView

from .models import Community, FeaturedCommunity, InclusionRequest


def _(x):
    """Identity function for string extraction."""
    return x


class CommunityModelView(ModelView):
    """ModelView for the Community."""

    can_create = True
    can_edit = True
    can_delete = False
    can_view_details = True
    column_display_all_relations = True
    form_columns = ('id', 'owner', 'title', 'description', 'page',
                    'ranking', 'fixed_points')
    column_list = (
        'id',
        'title',
        'owner.id',
        'deleted_at',
        'last_record_accepted',
        'ranking',
        'fixed_points',
    )
    column_searchable_list = ('id', 'title', 'description')


class FeaturedCommunityModelView(ModelView):
    """ModelView for the FeaturedCommunity."""

    can_create = True
    can_edit = True
    can_delete = True
    can_view_details = True
    column_display_all_relations = True
    column_list = (
        'community',
        'start_date',
    )


class InclusionRequestModelView(ModelView):
    """ModelView of the InclusionRequest."""

    can_create = False
    can_edit = False
    can_delete = True
    can_view_details = True
    column_list = (
        'id_community',
        'id_record',
        'expires_at',
        'id_user'
    )


community_adminview = dict(
    model=Community,
    modelview=CommunityModelView,
    category=_('Communities'),
)

request_adminview = dict(
    model=InclusionRequest,
    modelview=InclusionRequestModelView,
    category=_('Communities'),
)

featured_adminview = dict(
    model=FeaturedCommunity,
    modelview=FeaturedCommunityModelView,
    category=_('Communities'),
)



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2016 CERN.
#
# Invenio is free software; you can redistribute it
# and/or modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# Invenio is distributed in the hope that it will be
# useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Invenio; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA 02111-1307, USA.
#
# In applying this license, CERN does not
# waive the privileges and immunities granted to it by virtue of its status
# as an Intergovernmental Organization or submit itself to any jurisdiction.

"""Invenio module that adds support for communities."""

from __future__ import absolute_import, print_function

from invenio_indexer.signals import before_record_index
from sqlalchemy.event import listen
from werkzeug.utils import cached_property

from . import config
from .cli import communities as cmd
from .models import Community
from .permissions import permission_factory
from .receivers import create_oaipmh_set, destroy_oaipmh_set, \
    inject_provisional_community, new_request
from .signals import inclusion_request_created


class InvenioCommunities(object):
    """Invenio-Communities extension."""

    def __init__(self, app=None):
        """Extension initialization."""
        if app:
            self.init_app(app)

    def init_app(self, app):
        """Flask application initialization."""
        self.init_config(app)
        app.cli.add_command(cmd)
        app.extensions['invenio-communities'] = self
        # Register the jinja do extension
        app.jinja_env.add_extension('jinja2.ext.do')
        self.register_signals(app)

    def register_signals(self, app):
        """Register the signals."""
        before_record_index.connect(inject_provisional_community)
        if app.config['COMMUNITIES_OAI_ENABLED']:
            listen(Community, 'after_insert', create_oaipmh_set)
            listen(Community, 'after_delete', destroy_oaipmh_set)
        inclusion_request_created.connect(new_request)

    def init_config(self, app):
        """Initialize configuration."""
        app.config.setdefault(
            "COMMUNITIES_BASE_TEMPLATE",
            app.config.get("BASE_TEMPLATE",
                           "invenio_communities/base.html"))

        # Set default configuration
        for k in dir(config):
            if k.startswith("COMMUNITIES_"):
                app.config.setdefault(k, getattr(config, k))

    @cached_property
    def permission_factory(self):
        """Load default permission factory."""
        return permission_factory



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2016 CERN.
#
# Invenio is free software; you can redistribute it
# and/or modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# Invenio is distributed in the hope that it will be
# useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Invenio; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA 02111-1307, USA.
#
# In applying this license, CERN does not
# waive the privileges and immunities granted to it by virtue of its status
# as an Intergovernmental Organization or submit itself to any jurisdiction.

"""Community serializaiton."""

from __future__ import absolute_import, print_function

import json

from flask import current_app, request

from ..models import Community


def _format_args():
    """Get JSON dump indentation and separates."""
    # Ensure we can run outside a application/request context.
    try:
        pretty_format = \
            current_app.config['JSONIFY_PRETTYPRINT_REGULAR'] and \
            not request.is_xhr
    except RuntimeError:
        pretty_format = False

    if pretty_format:
        return dict(
            indent=2,
            separators=(', ', ': '),
        )
    else:
        return dict(
            indent=None,
            separators=(',', ':'),
        )


def community_responsify(schema_class, mimetype):
    """Create a community response serializer.

    :param serializer: Serializer instance.
    :param mimetype: MIME type of response.
    """
    def view(data, code=200, headers=None, links_item_factory=None,
             page=None, urlkwargs=None, links_pagination_factory=None):
        """Generate the response object."""
        if isinstance(data, Community):
            last_modified = data.updated
            response_data = schema_class(
                context=dict(item_links_factory=links_item_factory)
            ).dump(data).data
        else:
            last_modified = None
            response_data = schema_class(
                context=dict(
                    total=data.query.count(),
                    item_links_factory=links_item_factory,
                    page=page,
                    urlkwargs=urlkwargs,
                    pagination_links_factory=links_pagination_factory)
            ).dump(data.items, many=True).data

        response = current_app.response_class(
            json.dumps(response_data, **_format_args()),
            mimetype=mimetype)
        response.status_code = code

        if last_modified:
            response.last_modified = last_modified

        if headers is not None:
            response.headers.extend(headers)
        return response
    return view



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2015, 2016 CERN.
#
# Invenio is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# Invenio is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Invenio; if not, write to the Free Software Foundation, Inc.,
# 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
#
# In applying this license, CERN does not
# waive the privileges and immunities granted to it by virtue of its status
# as an Intergovernmental Organization or submit itself to any jurisdiction.

"""Invenio Communities configuration."""

from __future__ import unicode_literals

from datetime import timedelta

import pkg_resources

COMMUNITIES_REQUEST_EXPIRY_TIME = timedelta(days=365)
"""Time after which the inclusion requests automatically expire."""

COMMUNITIES_DELETE_HOLDOUT_TIME = timedelta(days=365)
"""Time after which the communities marked for deletion are hard-deleted."""

COMMUNITIES_LOGO_EXTENSIONS = ['png', 'jpg', 'jpeg']
"""Allowed file extensions for the communities logo."""

COMMUNITIES_RECORD_KEY = 'communities'
"""Key inside the JSON record for communities."""

COMMUNITIES_SORTING_OPTIONS = [
    'title',
    'ranking',
]
"""Possible communities sorting options."""

COMMUNITIES_DEFAULT_SORTING_OPTION = 'ranking'
"""Default sorting option."""

COMMUNITIES_OAI_FORMAT = 'user-{community_id}'
"""String template for the community OAISet 'spec'."""

COMMUNITIES_BUCKET_UUID = '00000000-0000-0000-0000-000000000000'
"""UUID for the bucket corresponding to communities."""

COMMUNITIES_INDEX_PREFIX = 'records-'
"""Key inside the JSON record for communities."""

COMMUNITIES_OAI_ENABLED = False
"""Using OAIServer if available."""
try:
    pkg_resources.get_distribution('invenio_oaiserver')
    COMMUNITIES_OAI_ENABLED = True
except pkg_resources.DistributionNotFound:  # pragma: no cover
    pass

COMMUNITIES_MAIL_ENABLED = False
"""Using Flask-Mail if available."""
try:
    pkg_resources.get_distribution('flask_mail')
    COMMUNITIES_MAIL_ENABLED = True
except pkg_resources.DistributionNotFound:  # pragma: no cover
    pass

COMMUNITIES_REQUEST_EMAIL_BODY_TEMPLATE = \
    'invenio_communities/request_email_body.html'
"""Template for InclusionRequest email body."""

COMMUNITIES_REQUEST_EMAIL_TITLE_TEMPLATE = \
    'invenio_communities/request_email_title.html'
"""Template for InclusionRequest email title."""

COMMUNITIES_REQUEST_EMAIL_SENDER = 'info@inveniosoftware.org'
"""Sender email for all inclusion request notification emails."""

COMMUNITIES_JSTEMPLATE_RESULTS_CURATE = \
    'templates/invenio_communities/ng_record_curate.html'
"""Angular template for records in curation view."""

COMMUNITIES_COMMUNITY_TEMPLATE = "invenio_communities/community_base.html"

COMMUNITIES_CURATE_TEMPLATE = "invenio_communities/curate.html"

COMMUNITIES_SEARCH_TEMPLATE = "invenio_communities/search.html"

COMMUNITIES_URL_COMMUNITY_VIEW = \
    '{protocol}://{host}/communities/{community_id}/'
"""String pattern to generate the URL for the view of a community."""



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2016 CERN.
#
# Invenio is free software; you can redistribute it
# and/or modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# Invenio is distributed in the hope that it will be
# useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Invenio; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA 02111-1307, USA.
#
# In applying this license, CERN does not
# waive the privileges and immunities granted to it by virtue of its status
# as an Intergovernmental Organization or submit itself to any jurisdiction.

"""Links for community serialization."""

from __future__ import absolute_import, print_function

from flask import current_app, request, url_for


def default_links_item_factory(community):
    """Factory for record links generation."""
    return dict(
        self=url_for(
            '.communities_item', community_id=community['id'], _external=True),
        html=current_app.config.get(
            'COMMUNITIES_URL_COMMUNITY_VIEW',
            '{protocol}://{host}/communities/{community_id}/'
        ).format(
            protocol=request.environ['wsgi.url_scheme'],
            host=request.environ['HTTP_HOST'],
            community_id=community['id']
        )
    )


def default_links_pagination_factory(page, urlkwargs):
    """Factory for record links generation."""
    endpoint = '.communities_list'

    links = {
        'self': url_for(endpoint, page=page.page, _external=True, **urlkwargs),
    }

    if page.has_prev:
        links['prev'] = url_for(endpoint, page=page.prev_num, _external=True,
                                **urlkwargs)
    if page.has_next:
        links['next'] = url_for(endpoint, page=page.next_num, _external=True,
                                **urlkwargs)

    return links
